#include "kk_sub_tsl.h"


typedef struct {
    void *mutex;
	int sub_devid;
    struct list_head dev_list;	
} kk_sub_dev_ctx_t;

static kk_sub_dev_ctx_t g_sub_dev_mgr = {0};
static kk_sub_dev_ctx_t * _sub_dev_get_ctx(void)
{
    return &g_sub_dev_mgr;
}
static int _sub_tsl_next_devid(void)
{
    kk_sub_dev_ctx_t *ctx = _sub_dev_get_ctx();

    return ctx->sub_devid++;
}

static void _sub_dev_mutex_lock(void)
{
    kk_sub_dev_ctx_t *ctx = _sub_dev_get_ctx();
    if (ctx->mutex) {
        kk_MutexLock(ctx->mutex);
    }
}

static void _sub_dev_mutex_unlock(void)
{
    kk_sub_dev_ctx_t *ctx = _sub_dev_get_ctx();
    if (ctx->mutex) {
        kk_MutexUnLock(ctx->mutex);
    }
}

int kk_tsl_init(void)
{
	char *tsl_str = NULL;
	int res = 0;
	kk_tsl_t *dev_shadow;
	sub_dev_node_t *node = NULL;
	kk_sub_dev_ctx_t *ctx = _sub_dev_get_ctx();
    ctx->mutex = kk_MutexCreate();
    if (ctx->mutex == NULL) {
        return FAIL_RETURN;
    }	
    ctx->sub_devid = 1;

    INIT_LIST_HEAD(&ctx->dev_list);	

	kk_sub_tsl_add("112233445566","a1h88DsZIaY");
	usleep(1000000);
	kk_test();
	return SUCCESS_RETURN;
}
int kk_sub_tsl_add(const char *device_mac,const char *product_type)
{
    sub_dev_node_t *node = NULL;
	kk_sub_dev_ctx_t *ctx = _sub_dev_get_ctx();
	char name[TSL_PATH_MAXLEN] = {0};
	char *tsl_str = NULL;
	int res = 0;
	if(device_mac == NULL||product_type == NULL)
	{
		return INVALID_PARAMETER;
	}

    node = malloc(sizeof(sub_dev_node_t));
    if (node == NULL) {
        return MEMORY_NOT_ENOUGH;
    }
	
	memset(node->product_type,0x0,sizeof(node->product_type));
	memcpy(node->device_mac, device_mac, strlen(device_mac));
	memcpy(node->product_type, product_type, strlen(product_type));	
	node->devid = _sub_tsl_next_devid();
	node->dev_shadow = NULL;
    memset(name,0x0,sizeof(name));
	sprintf(name,"%s.json",product_type);
	tsl_str = kk_load_json(name);
	if(tsl_str != NULL)
	{
		res = kk_tsl_create(tsl_str,strlen(tsl_str),&node->dev_shadow);
		free(tsl_str);
		if(res != 0){
			printf("[%s][%d]\n",__FUNCTION__,__LINE__);
			return FAIL_RETURN;
		}
	}
    INIT_LIST_HEAD(&node->linked_list);
    list_add_tail(&node->linked_list, &ctx->dev_list);
	return SUCCESS_RETURN;
}
int kk_sub_tsl_get_device_by_mac(_IN_ char device_mac[DEVICE_MAC_MAXLEN], _OU_ sub_dev_node_t **node)
{
    kk_sub_dev_ctx_t *ctx = _sub_dev_get_ctx();
    sub_dev_node_t *search_node = NULL;

    list_for_each_entry(search_node, &ctx->dev_list, linked_list, sub_dev_node_t) {
        if ((strlen(search_node->device_mac) == strlen(device_mac)) &&
            (memcmp(search_node->device_mac, device_mac, strlen(device_mac)) == 0)) {
            /* dm_log_debug("Device Found, Product Key: %s, Device Name: %s", product_key, device_name); */
            if (node) {
                *node = search_node;
            }
            return SUCCESS_RETURN;
        }
    }

    printf("Device Not Found, device_mac: %s\n", device_mac);
    return FAIL_RETURN;
}

int kk_sub_tsl_get_Identifier_by_index(_IN_ char device_mac[DEVICE_MAC_MAXLEN], int idx,_OU_ char **identifiers)
{
    int res = 0;
    sub_dev_node_t *node = NULL;
	int index = 0;
    kk_tsl_t *dev_shadow = NULL;	
	int propertiesNum = 0;
	kk_tsl_data_t *pProperty = NULL;
	char *identifier = NULL;
    res = kk_sub_tsl_get_device_by_mac(device_mac, &node);
    if (res != SUCCESS_RETURN) {
        return FAIL_RETURN;
    }
	dev_shadow = node->dev_shadow;
	propertiesNum = dev_shadow->property_number;
	if(idx >= propertiesNum){
		return FAIL_RETURN;
	}

	pProperty = (kk_tsl_data_t *)(dev_shadow->properties+idx);
	identifier = (char *)malloc(strlen(pProperty->identifier)+1);
	memset(identifier,0x0,strlen(pProperty->identifier)+1);
    memcpy(identifier,pProperty->identifier,strlen(pProperty->identifier));
	*identifiers = identifier;
    return SUCCESS_RETURN;
}
int kk_sub_tsl_get_Identifiers_Num(_IN_ char device_mac[DEVICE_MAC_MAXLEN],_OU_ int *num)
{
    int res = 0;
    sub_dev_node_t *node = NULL;
    kk_tsl_t *dev_shadow = NULL;	
    res = kk_sub_tsl_get_device_by_mac(device_mac, &node);
    if (res != SUCCESS_RETURN) {
        return FAIL_RETURN;
    }
	dev_shadow = node->dev_shadow;
	*num = dev_shadow->property_number;
    return SUCCESS_RETURN;
}
const char DM_MSG_REQUEST[] = "{\"LightStatus123\":\"1\"}";
int kk_test(void)
{
	int num;
	int i = 0;
	char *identifier  = NULL;	
	printf("[%s][%d]\n",__FUNCTION__,__LINE__);
	kk_test_fuc("112233445566",DM_MSG_REQUEST);

	return 0;
}

